@@ -47,6 +47,9 @@ void *js_alloc(int unit_count, int unit_size) {
     unsigned int hash_index,counter;
     lt_hash_spot *new,*point;
 #endif /* DEBUG */
+    /* Sanity check: Never allow this; makes C act buggy */
+    if(unit_size == 0 || unit_count == 0) 
+        return 0;
     data = (void *)malloc(unit_count * unit_size);
 #ifdef DEBUG
     /* If debug is set, we have a 64k element hash table which uses the low 
@@ -133,11 +136,11 @@ void *js_alloc_DEBUG(int unit_count, int unit_size, char *whence) {
     hash_index %= HASH_SIZE;
     /* Set up the string used to tell people why this memory was
        allocated */
-    if((why = calloc(js_strnlen(whence,2048) + 2)) == 0) {
+    if((why = malloc(strlen(whence) + 2)) == 0) {
         printf("Aieeee, mem alloc problem\n");
 	exit(64);
 	}
-    strncpy(why,whence,js_strnlen(whence,2048) + 1);
+    strncpy(why,whence,strlen(whence) + 1);
 #ifdef THREADS
     pthread_mutex_lock(&alloc_lock);
 #endif /* THREADS */
@@ -201,6 +204,8 @@ int js_dealloc(void *pointer) {
     lt_hash_spot *lpoint,*llast;
     unsigned int hash_index;
 #endif /* DEBUG */
+    if(pointer == 0) /* We don't bother with already dealloced memory */
+	    return 1;
 #ifdef DEBUG
     /* We need to find out the number of bytes that we have allocated 
        for this given address; subtract that from the total amount of
@@ -328,6 +333,80 @@ int js_show_stdout(js_string *js) {
     return 1;
     }
 
+/* show_esc_stdout: Display a csv1-compatible backslash escaped
+                    version of a given js_string object on standard output
+   input: Pointer to js_string object
+   output: -1 on failure, 1 on success */
+int show_esc_stdout(js_string *js) {
+    int counter = 0;
+    unsigned char this;
+
+    if(js_has_sanity(js) < 0)
+        return -1;
+
+    if(js->unit_size != 1) /* There is a bug which changes this while keeping
+                              the sanity */
+        return -1;
+    while(counter < js->unit_size * js->unit_count) {
+        this = *(js->string + counter);
+        if(this < 32 || this > 126) {
+            printf("\\%03o",this);
+            }
+        else if(this == '\\' || this == '%') {
+            printf("\\%c",this);
+            }
+        else {
+            putc(*(js->string + counter),stdout);
+            }
+        counter++;
+        }
+
+    return 1;
+    }
+
+/* safe_esc_stdout: Display a safe csv2-compatible backslash escaped
+                    version of a dlabel; this will make an
+		    hex escape sequence anything
+		    that is not [A-Za-z0-9\-\_] 
+   input: Pointer to js_string object
+   output: -1 on failure, 1 on success */
+int safe_esc_stdout(js_string *js) {
+    int counter = 0;
+    unsigned char this;
+
+    if(js_has_sanity(js) < 0)
+        return -1;
+
+    if(js->unit_size != 1) /* There is a bug which changes this while keeping
+                              the sanity */
+        return -1;
+    while(counter < js->unit_size * js->unit_count) {
+        this = *(js->string + counter);
+	/* A lot of the fetchzone code makes the first character a
+	 * space, which needs to be faithfully copied */
+	if(this == ' ' && counter == 0) {
+		printf(" ");
+	}
+	else if((this >= 'A' && this <= 'Z') || (this >= 'a' && this <= 'z') 
+	   || this == '-' || this == '_' || this > 127 /* UTF-8 */
+	   || this == '.' || this == '@' || (this >= '0' && this <= '9')) {
+            printf("%c",this);
+            }
+	/* We show a star only if it is the first character and the second
+         * character is a '.' character */
+	else if(this == '*' && counter == 0 && js->unit_count > 2 &&
+                *(js->string + 1) == '.') {
+	    printf("%c",this);
+	    }
+        else { 
+	    printf("\\x%02x",this); 
+	    }
+        counter++;
+        }
+
+    return 1;
+    }
+
 /* js_getline_stdin: Get a line from the standard input
    input: To to js_string object to put stdin contents in
    output: JS_ERROR on error, JS_SUCCESS on success.
@@ -487,8 +566,13 @@ int js_read(js_file *desc, js_string *js, int count) {
         return JS_ERROR;
     if(count % js->unit_size != 0)
         return JS_ERROR;
+#ifndef MINGW32
     if(count < 0 || count > SSIZE_MAX)
         return JS_ERROR;
+#else
+    if(count < 0 || count > 1024)
+        return JS_ERROR;
+#endif
 
     if(count > js->unit_size * js->max_count)
         return JS_ERROR;
@@ -526,8 +610,13 @@ int js_write(js_file *desc, js_string *js) {
         return JS_ERROR;
 
     to_write = js->unit_size * js->unit_count;
+#ifndef MINGW32
     if(to_write < 0 || to_write > SSIZE_MAX)
         return JS_ERROR;
+#else
+    if(to_write < 0 || to_write > 1024)
+        return JS_ERROR;
+#endif
 
     value = write(desc->file_desc,js->string,to_write);
 
@@ -745,7 +834,10 @@ int js_buf_getline(js_file *desc, js_string *js) {
    output: JS_ERROR on failure, JS_SUCCESS on success
    note: it's here because of strlen call */
 int js_qstr2js(js_string *js, char *string) {
-    return js_str2js(js,string,js_strnlen(string,1000000),js->unit_size);
+    if(js == 0) {
+	return JS_ERROR;
+        }
+    return js_str2js(js,string,strlen(string),js->unit_size);
     }
 
 /* js_adduint32: Add a 32-bit number to the end of a js_string obejct
@@ -810,22 +902,21 @@ u_int32_t js_readuint32(js_string *js, unsigned int offset) {
     }
 
 /* js_strnlen: Determine the length of a null-terminated string, up
-   to the length determined by the limit.  This is here because strnlen
-   is, alas, not a portable string library call.  This call is originally 
-   by Matthew T. Russotto */
-
+ * to the length determined by the limit.  This is here because strnlen
+ * is, alas, not a portable string library call.  This call is originally 
+ * by Matthew T. Russotto */
 int js_strnlen(char *s, u_int32_t limit) {
-    u_int32_t len;
-    if(limit > 2147483600 || limit < 0) { /* Just under 2 ** 31 */
-        return JS_ERROR;
-        }
-    if(s == NULL) {
-        return JS_ERROR;
-        }
-    len = 0; 
-    while(len < limit && (*s++)) {
-        len++;
-        }
-    return len; 
-    }
+	u_int32_t len;
+	if(limit > 2147483600 || limit < 0) { /* Just under 2 ** 31 */
+		return JS_ERROR;
+	}
+	if(s == NULL) {
+		return JS_ERROR;
+	}
+	len = 0; 
+	while(len < limit && (*s++)) {
+		len++;
+	}
+	return len;
+}
 
