@@ -5,6 +5,8 @@
 
    This is done with a hash that grows as needed for the data in question */
 
+#ifndef MARAHASH_INCLUDED
+#define MARAHASH_INCLUDED
 #ifndef JS_STRING_INCLUDED
 #include "JsStr.h"
 #endif
@@ -49,6 +51,7 @@ typedef struct {
 #define MARA_DNSRR 3 /* DNS RR object used by MaraDNS */
 #define MARA_DNS_NS 4 /* DNS record "closer" to what we are looking for */
 #define MARA_DNS_NEG 5 /* A cached instance of "this host is not there" */
+#define MARA_DNS_LIST 6 /* A list of RRs that share a given name */
 
 /* Function prototypes */
 /* mhash (assosciative array) objects */
@@ -59,14 +62,17 @@ int mhash_put(mhash *hash, js_string *key, void *value, int datatype);
 mhash_e mhash_get(mhash *hash, js_string *key);
 int mhash_put_js(mhash *hash, js_string *key,js_string *value);
 js_string *mhash_get_js(mhash *hash, js_string *key);
+int mhash_firstkey(mhash *hash, js_string *key);
+int mhash_nextkey(mhash *hash, js_string *key);
+
 /* mara_tuple (immutable list) objects */
 mara_tuple *mtuple_new(int elements);
 int mtuple_put(mara_tuple *tuple, js_string *js, int element);
 js_string *mtuple_get(mara_tuple *tuple, int element);
 mhash_e mhash_offset2val(mhash *hash, mhash_offset offset);
 js_string *mhash_get_immutable_key(mhash *hash, js_string *key);
-int mhash_add_ptr(mhash *hash, js_string *query, js_string *value);
 
 /* More function prototypes */
 #include "functions_MaraHash.h"
 
+#endif /* MARAHASH_INCLUDED */
