@@ -1,4 +1,4 @@
-/* Copyright (c) 2004 Sam Trenholme
+/* Copyright (c) 2004-2006 Sam Trenholme
  *
  * TERMS
  *
@@ -21,10 +21,18 @@
 #define FATAL_CSV2_READ -219
 /* Headers for a csv2_read state */
 
+typedef struct csv2_file {
+	FILE *reading;
+	char *filename;
+	struct csv2_file *next;
+} csv2_file;
+
 typedef struct csv2_read {
 	FILE *reading;
 	char *filename;
+	csv2_file *stack;
 	char context[19];
+	int stack_height;
 	int cplace;
 	int mnum;
 	int mplace;
