@@ -1,52 +1,36 @@
-/* Placed in the public domain 2000 by Sam Trenholme */
+/* Copyright (c) 2002,2003 Sam Trenholme
+ *
+ * TERMS
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * This software is provided 'as is' with no guarantees of correctness or
+ * fitness for purpose.
+ */
+
 /* Parse a mararc file */
 
 #include "../MaraDns.h"
 #include "../libs/MaraHash.h"
 #include "ParseCsv1_en.h"
 #include <stdlib.h>
+#ifndef MINGW32
 #include <pwd.h>
+#endif
 #include <sys/types.h>
 #include <stdio.h>
 
 /* Function prototypes */
 #include "../dns/functions_dns.h"
-extern int mara_goodjs();
-extern int js_space_chars();
-extern int ddip_2_ip();
-extern int js_changebyte();
-extern int js_numbers();
-
-/* show_esc_stdout: Display a csv1-compatible backslash escaped
-                    version of a given js_string object on standard output
-   input: Pointer to js_string object
-   output: -1 on failure, 1 on success */
-int show_esc_stdout(js_string *js) {
-    int counter = 0;
-    unsigned char this;
-
-    if(js_has_sanity(js) < 0)
-        return -1;
-
-    if(js->unit_size != 1) /* There is a bug which changes this while keeping
-                              the sanity */
-        return -1;
-    while(counter < js->unit_size * js->unit_count) {
-        this = *(js->string + counter);
-        if(this < 32 || this > 126) {
-            printf("\\%03o",this);
-            }
-        else if(this == '\\' || this == '%') {
-            printf("\\%c",this);
-            }
-        else {
-            putc(*(js->string + counter),stdout);
-            }
-        counter++;
-        }
-
-    return 1;
-    }
+#include "functions_parse.h"
 
 /* Parse a single line of a csv1 data file
    input: pointer to line of data, place to put the domain name (with
@@ -482,7 +466,7 @@ int ddip_2_ip(js_string *ddip, js_string *ip, int offset) {
             return JS_ERROR;
             }
         /* Place numbers in that string */
-        if(js_numbers(numdotq) == JS_ERROR) {
+        if(js_numbers(numdotq) == 0) {
             js_destroy(numdotq);
             js_destroy(dotq);
             numdotq = 0;
