@@ -31,7 +31,7 @@
 
 /* Keywords that are non-dictionary strings in Mara's rc file */
 
-#define KEYCOUNT 46
+#define KEYCOUNT 47
 
 char *keywords[KEYCOUNT] = {
         "bind_address", /* IPv4 Addresses to bind to (old name) */
@@ -123,6 +123,8 @@ char *keywords[KEYCOUNT] = {
                          * set while MaraDNS is running */
         "handle_noreply", /* How to handle the case of none of the remote
                            * servers replying at all */
+        "retry_cycles", /* The number of times we try to contact all of
+                         * the remote DNS servers to resolve a given name */
         "zone_transfer_acl" /* ACL of IPs allowed to perform zone transfers */
         };
 
@@ -884,11 +886,15 @@ int parseline(js_string *line, js_string *var, js_string *key,
         valend = quote2 - 1;
         /* We need to make sure there are no bare words between varend and
            the first quote, allowing 'foo bar = "baz"' to return an error */
-        if(ret != 2 && /* Always matches in key["foo"] = "bar" cases */
-           js_match_offset(alphanumeric,line,varend + 1) < quote1) {
-            js_qstr2js(var,"ERROR");
-            js_qstr2js(value,L_2ND_WORD); /* "Second bare word found before quotes" */
-            return JS_SUCCESS;
+        if(ret != 2) {
+            int q;
+            /* Always matches in key["foo"] = "bar" cases */
+            q = js_match_offset(alphanumeric,line,varend + 1);
+            if(q < quote1 && q != -2) {
+                js_qstr2js(var,"ERROR");
+                js_qstr2js(value,L_2ND_WORD); /* "Second bare word found before quotes" */
+                return JS_SUCCESS;
+                }
             }
         }
     /* If the value is unquoted, then look for bare numbers after = */
@@ -1157,7 +1163,18 @@ int read_mararc(js_string *fileloc,js_string *errorstr,int *errorret) {
             }
         else if(command == 1 && !error) { /* Add element to string object */
             tnum = keyword2num(var);
-	    if(tnum == -2 && var->unit_count > 0) {
+            if(tnum == -2) {
+                /* If an error happened parsing the line... */
+                js_qstr2js(tstr,"ERROR");
+                if(js_issame(tstr,var)) {
+                    if(!error) {
+                        error = linenum;
+                        *errorret = error;
+                        js_copy(value,errorstr);
+                        }
+                    }
+                }
+	    if(tnum == -2 && var->unit_count > 0 && !error) {
 		printf("FATAL ERROR: Unknown mararc variable ");
 		show_esc_stdout(var);
 		printf("\nPlease look for the uncommented string \"");
