@@ -48,6 +48,7 @@
 
 extern int log_level;
 extern int no_cname_warnings;
+int cname_warnings_shown = 0;
 
 
 /* Handler to handle warnings.
@@ -627,30 +628,90 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
 
     /* Create the js_String objects containing the zone name and
        corresponding filename */
-    if((udpzone = js_create(128,1)) == 0)
+    if((udpzone = js_create(128,1)) == 0) {
         return JS_ERROR;
-    if((filename = js_create(256,1)) == 0)
+        }
+    if((filename = js_create(256,1)) == 0) {
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((line = js_create(MAX_RECORD_LENGTH + 3,1)) == 0)
+        }
+    if((line = js_create(MAX_RECORD_LENGTH + 3,1)) == 0) {
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if(js_set_encode(line,MARA_LOCALE) == JS_ERROR)
+        }
+    if(js_set_encode(line,MARA_LOCALE) == JS_ERROR) {
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((pline = js_create(MAX_RECORD_LENGTH + 3,1)) == 0)
+        }
+    if((pline = js_create(MAX_RECORD_LENGTH + 3,1)) == 0) {
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if(js_set_encode(pline,MARA_LOCALE) == JS_ERROR)
+        }
+    if(js_set_encode(pline,MARA_LOCALE) == JS_ERROR) {
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((soaline = js_create(256,1)) == 0)
+        }
+    if((soaline = js_create(256,1)) == 0) {
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if(js_set_encode(soaline,MARA_LOCALE) == JS_ERROR)
+        }
+    if(js_set_encode(soaline,MARA_LOCALE) == JS_ERROR) {
+        js_destroy(soaline);
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((query = js_create(256,1)) == 0)
+        }
+    if((query = js_create(256,1)) == 0) {
+        js_destroy(soaline);
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((data = js_create(MAX_RECORD_LENGTH + 3,1)) == 0)
+        }
+    if((data = js_create(MAX_RECORD_LENGTH + 3,1)) == 0) {
+        js_destroy(query);
+        js_destroy(soaline);
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((zone = js_create(MAX_ZONE_SIZE,1)) == 0)
+        }
+    if((zone = js_create(MAX_ZONE_SIZE,1)) == 0) {
+        js_destroy(data);
+        js_destroy(query);
+        js_destroy(soaline);
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
-    if((desc = js_alloc(1,sizeof(js_file))) == 0)
+        }
+    if((desc = js_alloc(1,sizeof(js_file))) == 0) {
+        js_destroy(zone);
+        js_destroy(data);
+        js_destroy(query);
+        js_destroy(soaline);
+        js_destroy(pline);
+        js_destroy(line);
+        js_destroy(filename);
+        js_destroy(udpzone);
         return JS_ERROR;
+        }
 
     /* Pass 1: fill up the hash with the AN and NS answers */
 
@@ -660,6 +721,9 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
     if(csvs == 0) {
         /* A csv1 hash is no longer mandatory, now that MaraDNS has recursive
            capability */
+        js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+        js_destroy(pline); js_destroy(soaline); js_destroy(query);
+        js_destroy(data); js_destroy(zone); js_dealloc(desc);
         return 0;
         /*js_qstr2js(error,L_INITCSV); */
         /*"csv1 hash is not correctly initialized.\nMake sure to have csv1 initialized with csv1 = {}, and that it has one or\nmore elements." */
@@ -669,6 +733,9 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
     if(mhash_firstkey(csvs,zone) == 0) {
         /* We actually do not need to have anything in the hash, in
            the case of people running a recursive nameserver */
+        js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+        js_destroy(pline); js_destroy(soaline); js_destroy(query);
+        js_destroy(data); js_destroy(zone); js_dealloc(desc);
         return 0;
         }
     do {
@@ -677,10 +744,18 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
         if(zone != 0) {
 
             /* Make udpzone be like zone, but in raw RFC1035 format */
-            if(js_qstr2js(udpzone,"A") == JS_ERROR)
+            if(js_qstr2js(udpzone,"A") == JS_ERROR) {
+                js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+                js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                js_destroy(data); js_destroy(zone); js_dealloc(desc);
                 return JS_ERROR;
-            if(js_append(zone,udpzone) == JS_ERROR)
+                }
+            if(js_append(zone,udpzone) == JS_ERROR) {
+                js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+                js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                js_destroy(data); js_destroy(zone); js_dealloc(desc);
                 return JS_ERROR;
+                }
             if(hname_2rfc1035(udpzone) == JS_ERROR) {
                 js_qstr2js(error,L_BADZONE); /* "A zone file is incorrectly named.  All zone files must end with a dot, e.g.\ncsv1[\"example.com.\"] = \"filename\".\nBad zone name: " */
                 js_qappend("\ncsv1[\"",error);
@@ -688,16 +763,28 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
                 js_qappend("\"] = \"",error);
                 js_append(mhash_get_js(csvs,zone),error);
                 js_qappend("\"\n",error);
+                js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+                js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                js_destroy(data); js_destroy(zone); js_dealloc(desc);
                 return -2;
                 }
             /* Add the binary RR_NS to the end of the udpzone string
                (This makes authoritative checks easier) */
-            if(js_addbyte(udpzone,0) == JS_ERROR)
+            if(js_addbyte(udpzone,0) == JS_ERROR) {
+                js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+                js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                js_destroy(data); js_destroy(zone); js_dealloc(desc);
                 return JS_ERROR;
-            if(js_addbyte(udpzone,RR_NS) == JS_ERROR)
+                }
+            if(js_addbyte(udpzone,RR_NS) == JS_ERROR) {
+                js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+                js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                js_destroy(data); js_destroy(zone); js_dealloc(desc);
                 return JS_ERROR;
+                }
 
             /* Get the file name to open from the element's value */
+            js_destroy(filename);
             filename = mhash_get_js(csvs,zone);
             /* Open up the filename in question */
             if(js_open_read(filename,desc) == JS_ERROR) {
@@ -716,13 +803,22 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
 
                 /* Get the line */
                 result = js_buf_getline(desc,line);
-                if(result == JS_ERROR)
+                if(result == JS_ERROR) {
+                    js_destroy(udpzone); js_destroy(filename); 
+                    js_destroy(line);
+                    js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                    js_destroy(data); js_destroy(zone); js_dealloc(desc);
                     return JS_ERROR;
+                    }
                 else if(result == -2) {
                     printf("%s%d%s",L_LINE_NUMBER,linenum + 1,L_TOO_LONG);
                     show_esc_stdout(filename);
                     printf("\n");
                     /* "line number ... is too long in file " */
+                    js_destroy(udpzone); js_destroy(filename); 
+                    js_destroy(line);
+                    js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                    js_destroy(data); js_destroy(zone); js_dealloc(desc);
                     return JS_ERROR;
                     }
 
@@ -738,8 +834,15 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
                         continue;
                     /* Add the SOA in the special case of the zone
                        only having SOA and NS records */
-                    if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA)
+                    if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA) {
+
+                        js_destroy(udpzone); js_destroy(filename); 
+                        js_destroy(line);
+                        js_destroy(pline); js_destroy(soaline); 
+                        js_destroy(query);
+                        js_destroy(data); js_destroy(zone); js_dealloc(desc);
                         return JS_ERROR;
+                        }
                     add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
                     break;
                     }
@@ -749,8 +852,13 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
                     break;
 
                 /* Process the % character and any \ sequences */
-                if(bs_process(line,pline,zone) == JS_ERROR)
+                if(bs_process(line,pline,zone) == JS_ERROR) {
+                    js_destroy(udpzone); js_destroy(filename); 
+                    js_destroy(line);
+                    js_destroy(pline); js_destroy(soaline); js_destroy(query);
+                    js_destroy(data); js_destroy(zone); js_dealloc(desc);
                     return JS_ERROR;
+                    }
 
                 /* Parse the line and get the rr type of the query */
                 rr_type = parse_csv1_line(pline,query,data,&ttl);
@@ -794,8 +902,15 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
                        record knows what NS records to place in the NS
                        section of the answer) */
                     else if(rrnum == 1) {
-                        if(js_copy(pline,soaline) == JS_ERROR)
+                        if(js_copy(pline,soaline) == JS_ERROR) {
+                            js_destroy(udpzone); js_destroy(filename); 
+                            js_destroy(line);
+                            js_destroy(pline); js_destroy(soaline); 
+                            js_destroy(query);
+                            js_destroy(data); js_destroy(zone); 
+                            js_dealloc(desc);
                             return JS_ERROR;
+                            }
                         }
                     /* Only one SOA allowed per zone */
                     else if(rrnum != 1 && rr_type == RR_SOA) {
@@ -812,8 +927,16 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
                     else if((rr_type != RR_NS || !js_issame(query,udpzone))
                             && in_ns) {
                         /* Add the SOA after the zone's NS records */
-                        if(parse_csv1_line(soaline,query,data,&ttl) != RR_SOA)
+                        if(parse_csv1_line(soaline,query,data,&ttl) != 
+			   RR_SOA) {
+                            js_destroy(udpzone); js_destroy(filename); 
+                            js_destroy(line);
+                            js_destroy(pline); js_destroy(soaline); 
+                            js_destroy(query);
+                            js_destroy(data); js_destroy(zone); 
+                            js_dealloc(desc);
                             return JS_ERROR;
+                            }
                         add_rr_to_bighash(maintable,query,data,ttl,udpzone,0);
                         /* Now, add the record we are looking at (the
                            first non-SOA and non-authoritative-NS record) */
@@ -845,6 +968,9 @@ int parse_csv1s(mhash *maintable, js_string *error, int recursive) {
             js_close(desc);
             }
         } while(mhash_nextkey(csvs,zone) != 0);
+    js_destroy(udpzone); js_destroy(filename); js_destroy(line);
+    js_destroy(pline); js_destroy(soaline); js_destroy(query);
+    js_destroy(data); js_destroy(zone); js_dealloc(desc);
     return JS_SUCCESS;
 }
 
@@ -1030,12 +1156,19 @@ int add_an(mhash *bighash, js_string *error) {
                         if(record != 0 && adata.value != 0)
                             record->list = adata.value;
                         }
-		    else if(no_cname_warnings == 0) {
+		    else if(no_cname_warnings == 0 && 
+			       cname_warnings_shown < 10) {
 			printf("Warning: The CNAME record ");
 			human_readable_dns_query(query,1);
 			printf(" is a dangling CNAME record.\n"
 "Please read the FAQ entry about FAQ entry about dangling CNAME records.\n");
+			cname_warnings_shown++;
 		        }
+		    else if(no_cname_warnings == 0 &&
+			      cname_warnings_shown == 10) {
+		       printf("Warning: More dangling CNAME records exist.\n");
+                       cname_warnings_shown++;
+                       }
                     } while ((record = record->next) != 0);
 	        }
         } while(mhash_nextkey(bighash,query) != 0);
