@@ -1,4 +1,20 @@
-/* Placed in the public domain 2001 by Sam Trenholme */
+/* Copyright (c) 2002,2003 Sam Trenholme
+ *
+ * TERMS
+ *
+ * Redistribution and use in source and binary forms, with or without
+ * modification, are permitted provided that the following conditions
+ * are met:
+ *
+ * 1. Redistributions of source code must retain the above copyright
+ *    notice, this list of conditions and the following disclaimer.
+ * 2. Redistributions in binary form must reproduce the above copyright
+ *    notice, this list of conditions and the following disclaimer in the
+ *    documentation and/or other materials provided with the distribution.
+ *
+ * This software is provided 'as is' with no guarantees of correctness or
+ * fitness for purpose.
+ */
 
 /* This is a version of askmara modified to test a name server by sending an
    AAAA query to the nameserver in question and seeing what we get.
@@ -26,8 +42,8 @@
 
 u_int16_t gen_id(char *hostname) {
     /* May as well bring out the bug guns (e.g. our rng) */
-    BYTE r_inBlock[17],r_outBlock[17],r_binKey[34];
-    BYTE r_keyMaterial[320];
+    MARA_BYTE r_inBlock[17],r_outBlock[17],r_binKey[34];
+    MARA_BYTE r_keyMaterial[320];
     keyInstance r_keyInst;
     cipherInstance r_cipherInst;
     unsigned char crypto_key[34];
