@@ -542,6 +542,10 @@ int csv2_compatible_output(js_string *uindata, js_string *query) {
     if(read_hdr(uindata,&header) == JS_ERROR)
         harderror(L_INHEADER_CONV); /* Problem converting inheader */
 
+    if(header.tc == 1) {
+	printf("# Remote server said: TRUNCATED\n");
+    }
+
     /* If not 0, show them the rcode from the remote server */
     switch(header.rcode) {
 	case 1:
